Class WebReportsCGISession of CGISession from "Webclass.cc"

/*    

   WebReports.cc provides a generic method for printing html reports.            
  
   Authors: Michael Nuwer and John Staub (06/28/2000)

   Updated by Michael Nuwer (06/28/2003) 
   Added:  Support for groupby
           Alternating row shading
           Support of drill down reports
           Support for SQL files
           Support for Web Report Wizard

*/

/// Properties for <TABLE></TABLE>
this.TableAlign       ="DEFAULT" 
this.TableBGColor     ="DEFAULT" 
this.TableBorder      ="DEFAULT" 
this.TableCellpadding ="DEFAULT" 
this.TableCellspacing ="DEFAULT" 
this.TableWidth       ="DEFAULT" 
this.TableHSpace      ="DEFAULT"
this.TableVSpace      ="DEFAULT"


/// Properties for the Table Data rows <TD></TD>
this.aTDAlign   = new array()  // "CENTER|LEFT|RIGHT"
this.aTDValign  = new array()  // "BASELINE|BOTTOM|MIDDLE|TOP"
this.aTDBGColor = new array()  // "color" 
this.aTDWidth   = new array()  // "pixelWidth|PecentOfTable"
this.aTDNowrap  = new array()  // "value" 


/// Properties for header row <TH></TH>
this.aTHAlign   = new array()  // "CENTER|LEFT|RIGHT"
this.aTHValign  = new array()  // "BASELINE|BOTTOM|MIDDLE|TOP"
this.aTHBGColor = new array()  // "color" 
this.aTHWidth   = new array()  // "pixelWidth|PecentOfTable"
this.aTHNowrap  = new array()  // "value" 

/// Font properties
this.TDFontName    ="DEFAULT"  // Name of typeface
this.TDFontColor   ="DEFAULT"  // #rrggbb 
this.TDFontSize    ="DEFAULT" 
this.TDFontBold    ="DEFAULT"  // Yes
this.TDFontItalic  ="DEFAULT"  // Yes

this.THFontName    ="DEFAULT"  // Name of typeface
this.THFontColor   ="DEFAULT"  // #rrggbb 
this.THFontSize    ="DEFAULT"
this.THFontBold    ="Yes"      // Yes
this.THFontItalic  ="DEFAULT"  // Yes


this.GroupBy        = false
this.printFooter    = false
this.drillDown      = false
this.DrillDownLabel = "FIELDVALUE"

this.bAlternateShading = false
this.AlternateColor = "#FFFFFF"
this.homePage = "http://your.home.com"

/////////////////////////////////////////////////
/// Initialize the custom property arrays
/////////////////////////////////////////////////
function Init(aFieldArray)
   local rs
   rs = this.q.rowset

   ////////////////////////////////////////  
   /// Set up the field-width array 
   ////////////////////////////////////////
   nTotal = 0
   for n = 1 to aFieldArray.size
     nTotal = nTotal + rs.fields[aFieldArray[n]].length
   next
   
   ////////////////////////////////////////////////////////
   ///  We want to use integers for the column widths.
   ///  The cell widths for HTML tables should 
   ///  always add up to 100%.  So the width of the last
   ///  column is a residual.
   ////////////////////////////////////////////////////////
   
   nCumPercent = 0     ////  used to keep a running total of column widths.
  
   for n = 1 to aFieldArray.size - 1   ///  the last column will be the residual.
      cPercent = str( (rs.fields[aFieldArray[n]].length/nTotal) *100)
      nCumPercent = nCumPercent + val(cPercent)
      this.aTDWidth.add( ltrim(cPercent) + "%" )
      this.aTHWidth.add( ltrim(cPercent) + "%" )
   next
   this.aTDWidth.add( ltrim(str(100 - nCumPercent)) + "%" )
   this.aTHWidth.add( ltrim(str(100 - nCumPercent)) + "%" )
   
   for n = 1 to aFieldArray.size
      this.aTDAlign.add("DEFAULT")      
      this.aTDValign.add("DEFAULT")    
      this.aTDBGColor.add("DEFAULT")
      this.aTDNowrap.add("DEFAULT")

      this.aTHAlign.add("DEFAULT")
      this.aTHValign.add("DEFAULT")  
      this.aTHBGColor.add("#CCCCCC")
      this.aTHNowrap.add("DEFAULT")
         
   next                 
   return


//////////////////////////////////////////////////
///  Create the query object. 
///  Vaify that that the search value exists.
///  Return the result.
//////////////////////////////////////////////////

function Lookup()

   if type("this.dataBase") # "U" and not empty(this.dataBase)
      this.db = new database()
      this.db.databasename = this.Database
      this.db.active = true
   endif

   this.q = new query()
   if type("this.dB") # "U"  
      this.q.database = this.db
   endif

   // Data source can be a SQLString or a table
   if type("this.sqlString") # "U"  and not empty(this.sqlString)
      this.q.sql := this.sqlString
      //  sqlString cannot have an index
   else
      this.q.sql := 'select * from "' + this.Table + '"'
   endif

   this.q.active = true

   if type("this.indexName") # "U"   and not empty(this.indexName)
      this.q.rowset.indexname := this.IndexName
   endif

   //  if there is a property named "RangeValue"
   if type("this.RangeValue") # "U"   and not empty(this.RangeValue)
      this.q.rowset.setrange( this.RangeValue ) 
   // if your index key is based upon uses all upper case
   // then be sure "RangeValue" is also upper case 
   endif    										     

   //  if there is a property named "filter"
   if type("this.filter") # "U"  and not empty(this.filter) 
      // Match partial length and ignore case
      this.q.rowset.filterOptions = 3   

      // Set the filter
      this.q.rowset.filter = this.Filter
   endif

   Return this.q.rowset.first() 

//////////////////////////////////////////////////////////////////
/// ResponsePageTop will print the html that is located before 
/// the report table.  This could be the page title and logo, the  
/// left navigation stuff, etc.  This material is often the same
/// on all page at a web site.  The old dBASE site for example.
/// If a site uses frames, this method may not be needed.
/////////////////////////////////////////////////////////////////

Function ResponsePageTop(Title)
      /////// Stream out header
      
      cTitle = iif(empty(Title),'Response Page',Title)

      with (this.fOut)
         puts('Content-type: text/html')
         puts('')
         puts('<HTML>')
         puts('')
         puts('<HEAD>')
         puts('	<META HTTP-EQUIV="Content-Type" CONTENT="text/html">')
         puts('	<TITLE> '+cTitle+'</TITLE>')
         puts('</HEAD>')
         puts('')
         ////// Start page body   
         puts('<BODY BGCOLOR="#FFFFFF">')
         puts('')
	      puts('<h3><Center>' + cTitle + '</Center></h3>')
      endwith
      return

//////////////////////////////////////////////////////////////////
/// ResponsePageTable prints the table and its data.
//////////////////////////////////////////////////////////////////

Function ResponsePageTable(aCaptionArray,aFieldArray)

   this.aCaption = aCaptionArray
   this.aFields = aFieldArray

   local rs,nRow
   local cTableAlign, cTableBGColor, cTableBorder
   local cTableCellpadding, cTableCellspacing, cTableWidth,cTableHSpace,cTableVSpace
   rs = this.q.rowset

   with (this.fOut)

       ////////////////////////////////////////////
       // Print a table for the report. The first
       // row is the Heading Captions. 
	    ////////////////////////////////////////////

       cTableAlign = iif(this.TableALIGN="DEFAULT","",[ALIGN="] + this.TableALIGN + [" ]) 
       cTableBGColor = iif(this.TableBGColor="DEFAULT","",[BGCOLOR = "] + this.TableBGColor + [" ])  
       cTableBorder = iif(this.TableBORDER="DEFAULT","",[BORDER = "] + this.TableBORDER + [" ])  
       cTableCellpadding = iif(this.TableCELLPADDING="DEFAULT","",[CELLPADDING="] + this.TableCELLPADDING + [" ] )  
       cTableCellspacing = iif(this.TableCELLSPACING="DEFAULT","",[CELLSPACING="] + this.TableCELLSPACING + [" ])  
       cTableWidth = iif(this.TableWIDTH="DEFAULT","",[WIDTH="] + this.TableWIDTH + [" ])  
       cTableHSpace = iif(this.TableHSpace="DEFAULT","",[HSPACE="] + this.TableHSpace + [" ]) 
       cTableVSpace = iif(this.TableVSpace="DEFAULT","",[HVSPACE="] + this.TableVSpace + [" ]) 

       puts([<TABLE ] + cTableAlign + cTableBGColor + cTableBorder + cTableCellpadding +;
                     cTableCellspacing + cTableWidth + cTableHSpace + cTableVSpace + [>])  

    endwith
    if this.GroupBy

       this.printGroupHeader()
       VarGroupBy = rs.fields[this.GroupField].value
       this.GroupCounter = 0
    endif


    with (this.fOut)
       puts([<TR>])
   
	    for i = 1 to this.aCaption.size
	       cCaption = trim(this.aCaption[i])

          cTHWidth  = [WIDTH="] + this.aTDWidth[i] + [" ]
          cTHAlign = iif(this.aTHAlign[i] = "DEFAULT","",[ALIGN="] + this.aTHAlign[i] + [" ])
          cTHValign = iif(this.aTHVAlign[i] = "DEFAULT","",[VALIGN="] + this.aTHValign[i] + [" ])
          cTHBGColor = iif(this.aTHBGColor[i] = "DEFAULT","",[BGCOLOR="] + this.aTHBGColor[i] + [" ])
          cTHNowrap = iif(this.aTHNowrap[i] = "DEFAULT","",[NOWRAP ])

          cTHFontName = iif( this.THFontName="DEFAULT","",[FACE="] + this.THFontName + [" ])   
          cTHFontColor = iif( this.THFontColor="DEFAULT","",[COLOR="] + this.THFontColor + [" ])    
          cTHFontSize = iif( this.THFontSize="DEFAULT","",[SIZE="] + this.THFontSize + [" ])

          cTHFontTagOpen = ""
          cTHFontTagClose = ""
          if len(trim(cTHFontName))+len(trim(cTHFontColor))+len(trim(cTHFontSize)) > 0
             cTHFontTagOpen = [<FONT ] + cTHFontName + cTHFontColor + cTHFontSize + [>]
             cTHFontTagClose = [</FONT>]
          endif

          cTHFontBoldOpen = iif( this.THFontBold="DEFAULT","",[<B>])     
          cTHFontBoldClose = iif( this.THFontBold="DEFAULT","",[</B>] )
          cTHFontItalicOpen = iif( this.THFontItalic="DEFAULT","",[<I>] )   
          cTHFontItalicClose = iif( this.THFontItalic="DEFAULT","",[</I>]) 


          ///  Note: We are using the TD tag (not the TH tag) for the heading
          ///  This gives the user more control of the format
          ///  The property names, however, ust "TH"
          puts([<TD ] + cTHWidth + cTHAlign + cTHValign + cTHBGColor + cTHNowrap + [>]+ ;
	            cTHFontTagOpen + cTHFontBoldOpen + cTHFontItalicOpen ;
                + cCaption  + ;
                cTHFontItalicClose + cTHFontBoldClose + cTHFontTagClose + [</TD>])

	    next
 
	    puts([</TR>])

	    ////////////////////////////////////
	    /// Print one row for each record
	    ////////////////////////////////////
       nRow = 1
	    for n = 1 to rs.count()
	       puts([<TR>])
           
          //////////////////////////////////////////////////
	       ///  print one cell for each field.
	       /////////////////////////////////////////////////
	       for i = 1 to this.aFields.size
	      
	          ////////////////////////////////////////////
	          /// Grab the field name to be printed.
	          ////////////////////////////////////////////

	          cFieldName = this.aFields[i]
	      
	          /////////////////////////////////////////////
	          /// GetFieldValue is Alan's code, adapted from
	          /// the base class (WebClass.cc).
	          /////////////////////////////////////////////

	          cFieldValue = trim( CLASS::GetFieldValue(cFieldName) )

             cTDWidth = [WIDTH="] + this.aTDWidth[i] + [" ] 
             cTDAlign = iif(this.aTDAlign[i] = "DEFAULT","",[ALIGN="] + this.aTDAlign[i] + [" ])
             cTDValign = iif(this.aTDVAlign[i] = "DEFAULT","",[VALIGN="] + this.aTDValign[i] + [" ])
             cTDBGColor = iif(this.aTDBGColor[i] = "DEFAULT","",[BGCOLOR="] + this.aTDBGColor[i] + [" ])
             cTDNowrap = iif(this.aTDNowrap[i] = "DEFAULT","",[NOWRAP ])

             cTDFontName = iif( this.TDFontName="DEFAULT","",[FACE="] + this.TDFontName + [" ])   
             cTDFontColor = iif( this.TDFontColor="DEFAULT","",[COLOR="] + this.TDFontColor + [" ])    
             cTDFontSize = iif( this.TDFontSize="DEFAULT","",[SIZE="] + this.TDFontSize + [" ])

             cTDFontTagOpen = ""
             cTDFontTagClose = ""
             if len(trim(cTDFontName))+len(trim(cTDFontColor))+len(trim(cTDFontSize)) > 0
                cTDFontTagOpen = [<FONT ] + cTDFontName + cTDFontColor + cTDFontSize + [>]
                cTDFontTagClose = [</FONT>]
             endif

             cTDFontBoldOpen = iif( this.TDFontBold="DEFAULT","",[<B>])     
             cTDFontBoldClose = iif( this.TDFontBold="DEFAULT","",[</B>] )
             cTDFontItalicOpen = iif( this.TDFontItalic="DEFAULT","",[<I>] )   
             cTDFontItalicClose = iif( this.TDFontItalic="DEFAULT","",[</I>]) 

             ////////   If drill down report   /////////////
             cTDAnchorOpen  = ""
             cTDAnchorClose = ""
             if this.drillDown and (i = 1)  /// this.DrillDownField)

                cDrillDownField = this.drillDownField
                cIDValue = ltrim(trim( CLASS::GetFieldValue(cDrillDownField) ) )
                cIDValue = this.escapeURL( cIDValue )
                 
                cTDAnchorOpen  = [<A HREF="] + this.DrillDownCommand + cIDValue + [">] 
                cTDAnchorClose = [</A>]

                if this.DrillDownLabel # "FIELDVALUE"
                   cFieldValue = this.DrillDownLabel
                endif
             endif
             //////////////////////////                     

             if this.bAlternateShading and mod(nRow,2)==0
                cTDBGColor = [BGCOLOR="]+this.AlternateColor+[" ]
             endif

             puts([<TD ] + cTDAlign + cTDValign + cTDBGColor + cTDNowrap + [>]+ ;
                 cTDFontTagOpen + cTDFontBoldOpen + cTDFontItalicOpen + ;
                 cTDAnchorOpen + ;
	              cFieldValue + ;
                 cTDAnchorClose + ;
                 cTDFontItalicClose + cTDFontBoldClose + cTDFontTagClose + [</TD>])

	       next
          nRow ++
	   
	       ////////////////////////
	       /// End the row
	       ///////////////////////
	       puts([</TR>])

          //  Counters and accumulators go here
          //  Subclass this class and add a method named
          //  Aggragate(). The method should count or
          //  accumulate any of the fields in the aFieldsArray.
          //  Store the values in custom properties (this.Counter1),
          //  then override the printGroupFooter method in your sub class
          //  In that method stream your counters                    

          if type("this.aggragate") == "FP"  and this.GroupBy

             this.aggragate()

          endif

          rs.next()

          if this.GroupBy and ( rs.fields[this.GroupField].value <> VarGroupBy ) 


             if this.printFooter
                rs.next(-1)
                this.printGroupFooter()
                rs.next()
             endif

             if (not rs.endOfSet)

                this.printGroupHeader(true)

                VarGroupBy = rs.fields[this.GroupField].value

                //  Reset alternate shading counter
                nRow = 1

             endif
          endif

       next

       ////////////////////////
	    /// End the Table
	    ///////////////////////

	    puts([</TABLE>])
       puts([<BR>])
         
   endwith
   return





//////////////////////////////////////////////////////////////////////
///  ResponsePageBottom prints any html that would be at the bottom
///  of a page (e.g. copyrights) and closes the remaining html tags.
//////////////////////////////////////////////////////////////////////

Function ResponsePageBottom()
   with (this.fOut)	 
       puts('<BR>')
       puts('<CENTER><a href="javascript:history.go(-1)"><FONT SIZE="3" FACE="Arial,Helvetica">Click here to try your query again</FONT></CENTER></A>')
       puts('<BR>')
       puts('<CENTER><a href="' + this.homePage +'"><FONT SIZE="3" FACE="Arial,Helvetica">Home</FONT></CENTER></A>')


       puts('   ')
       puts([</BODY>])
       puts([ ])
       puts([</HTML>])

   endwith

   return



function PrintGroupHeader(PrintCaption)
   local rs

   PrintCaption = iif(empty(PrintCaption),false,true)
   rs = this.q.rowset

   cTHBGColor = iif(this.aTHBGColor[1] = "DEFAULT","",[BGCOLOR="] + this.aTHBGColor[1] + [" ])
   cTHFontName = iif( this.THFontName="DEFAULT","",[FACE="] + this.THFontName + [" ])   
   cTHFontColor = iif( this.THFontColor="DEFAULT","",[COLOR="] + this.THFontColor + [" ])    
   cTHFontSize = iif( this.THFontSize="DEFAULT","",[SIZE="] + this.THFontSize + [" ])

   //  determine size of font to use
   //  Use one relative size larger than column heading size
   cSize = iif( this.THFontSize="DEFAULT","3",this.THFontSize)
   do case
   case left(cSize,1)="+"
      cSize = val(cSize)+1
      cSize = "+"+ltrim(str(cSize,1,0))
   case left(cSize,1)="-"
      cSize = val(cSize)+1
      cSize = "-"+ltrim(str(cSize,1,0))
   otherwise
      cSize = val(cSize)+1
      cSize = ltrim(str(cSize,1,0))
   endcase
   cSize =[SIZE="] + cSize + [" ]

   cTHFontTagOpen = ""
   cTHFontTagClose = ""
   if len(trim(cTHFontName))+len(trim(cTHFontColor))+len(trim(cTHFontSize)) > 0
      cTHFontTagOpen = [<FONT ] + cTHFontName + cTHFontColor + cSize + [>]
      cTHFontTagClose = [</FONT>]
   endif
      
   cValue = this.GetFieldValue(this.GroupField)
   with (this.fOut)     

      /// print header cell
      puts([<TR><TD ] + cTHBGColor + [ COLSPAN="]+ltrim(str(this.aFields.size))+[">] )
	   puts(cTHFontTagOpen + [<P>&nbsp;<BR><B>]+cValue+[</B>]  + ;
           cTHFontTagClose + [</TD></TR>])
      
      if PrintCaption             
         //  Print column headings
         puts([<TR>])
   
	      for i = 1 to this.aCaption.size
	         cCaption = trim(this.aCaption[i])

            cTHWidth  = [WIDTH="] + this.aTDWidth[i] + [" ]
            cTHAlign = iif(this.aTHAlign[i] = "DEFAULT","",[ALIGN="] + this.aTHAlign[i] + [" ])
            cTHValign = iif(this.aTHVAlign[i] = "DEFAULT","",[VALIGN="] + this.aTHValign[i] + [" ])
            cTHBGColor = iif(this.aTHBGColor[i] = "DEFAULT","",[BGCOLOR="] + this.aTHBGColor[i] + [" ])
            cTHNowrap = iif(this.aTHNowrap[i] = "DEFAULT","",[NOWRAP ])

            cTHFontName = iif( this.THFontName="DEFAULT","",[FACE="] + this.THFontName + [" ])   
            cTHFontColor = iif( this.THFontColor="DEFAULT","",[COLOR="] + this.THFontColor + [" ])    
            cTHFontSize = iif( this.THFontSize="DEFAULT","",[SIZE="] + this.THFontSize + [" ])

            cTHFontTagOpen = ""
            cTHFontTagClose = ""
            if len(trim(cTHFontName))+len(trim(cTHFontColor))+len(trim(cTHFontSize)) > 0
               cTHFontTagOpen = [<FONT ] + cTHFontName + cTHFontColor + cTHFontSize + [>]
               cTHFontTagClose = [</FONT>]
            endif

            cTHFontBoldOpen = iif( this.THFontBold="DEFAULT","",[<B>])     
            cTHFontBoldClose = iif( this.THFontBold="DEFAULT","",[</B>] )
            cTHFontItalicOpen = iif( this.THFontItalic="DEFAULT","",[<I>] )   
            cTHFontItalicClose = iif( this.THFontItalic="DEFAULT","",[</I>]) 

            puts([<TD ] + cTHWidth + cTHAlign + cTHValign + cTHBGColor + cTHNowrap + [>]+ ;
	           cTHFontTagOpen + cTHFontBoldOpen + cTHFontItalicOpen ;
               + cCaption  + ;
               cTHFontItalicClose + cTHFontBoldClose + cTHFontTagClose + [</TD>])

	      next
         //  Close row
         puts([</TR>])
         
      endif
   endwith
   return

function PrintGroupFooter()
   cTHBGColor = iif(this.aTDBGColor[1] = "DEFAULT","",[BGCOLOR="] + this.aTDBGColor[1] + [" ])
   cTHFontName = iif( this.THFontName="DEFAULT","",[FACE="] + this.THFontName + [" ])   
   cTHFontColor = iif( this.THFontColor="DEFAULT","",[COLOR="] + this.THFontColor + [" ])    
   cTHFontSize = iif( this.THFontSize="DEFAULT","",[SIZE="] + this.THFontSize + [" ])

   cTHFontTagOpen = ""
   cTHFontTagClose = ""
   if len(trim(cTHFontName))+len(trim(cTHFontColor))+len(trim(cTHFontSize)) > 0
      cTHFontTagOpen = [<FONT ] + cTHFontName + cTHFontColor + cSize + [>]
      cTHFontTagClose = [</FONT>]
   endif
      
   with (this.fOut)

      /// print footer cell
      puts([<TR><TD ] + cTHBGColor + [ COLSPAN="]+ltrim(str(this.aFields.size))+[">] )
	   puts(cTHFontTagOpen + [<P><B>Count for group: ]+this.GroupCounter+[</B>]  + ;
           cTHFontTagClose + [</TD></TR>])
      
   endwith

   // reset the group row counter
   this.GroupCounter = 0 
   return

function Aggragate

   // Increment the group row counter
   this.GroupCounter ++ 
   return


Function GetFieldValue(cFieldName)
///////////////////////////////////////////
/// This method converts the field value 
/// to a string and returns it.  Adapted 
/// from A.A. Katz's LoadArrayFromFields().
///////////////////////////////////////////

   local cFieldValue

	rf = this.q.rowset.fields[cFieldName]   
	    
	// if field exists
   if type( 'rf' ) = 'O' 

      with ( rf )

         // if field is not empty
         if empty(value) and type # 'LOGICAL' 

            cFieldValue = "&nbsp;"

         else  
            // figure out its type and store it as value to array
            if type = 'AUTOINC' or type = 'NUMERIC'  or  ;
               type = 'LONG' or type = 'INTEGER'  or ;
               type = 'DOUBLE' or type = 'FLOAT'

               cFieldValue = str(value,length,decimalLength)
          
            elseif type = 'DATE'
               cFieldValue = dtoc(value)

            elseif type = 'LOGICAL'
               cFieldValue = iif(value,'TRUE','FALSE')

            elseif type = 'BINARY' or type = 'OLE'
               // do nothing
               cFieldValue = "&nbsp;"
            else // char or memo

               cFieldValue = value
            endif
         endif
      endwith
   endif
   return cFieldValue

function escapeURL(cData)
   //   Bowen Moursund (dBASE, Inc.)
   //   March 2000
   local cRetStr, nLen, cChar, i
   cRetStr = ""
   nLen = len(cData)
   for i = 1 to nLen
      cChar = substr(cData, i, 1)
      if cChar = " "
         cRetStr += "+"
      elseif not cChar $ "&=0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
         cRetStr += "%"+itoh(asc(cChar))
      else
         cRetStr += cChar
      endif
   next i
   return cRetStr

endClass
